Package: SimDesign
Title: Structure for Organizing Monte Carlo Simulation Designs
Version: 2.27
Authors@R: c(person("Phil", "Chalmers", email = "rphilip.chalmers@gmail.com", role = c("aut", "cre"),
                    comment = c(ORCID="0000-0001-5332-2810")),
             person("Matthew", "Sigal", role = c("ctb")),
             person("Ogreden", family="Oguzhan", role = c("ctb")),
             person("Mikko", family="Rönkkö", role = c("aut")),
             person("Moritz", family="Ketzer", role = c("ctb")))
Description: Provides tools to safely and efficiently organize and execute 
    Monte Carlo simulation experiments in R.
    The package controls the structure and back-end of Monte Carlo simulation experiments
    by utilizing a generate-analyse-summarise workflow. The workflow safeguards against 
    common simulation coding issues, such as automatically re-simulating non-convergent results, 
    prevents inadvertently overwriting simulation files, catches error and warning messages
    during execution, implicitly supports parallel processing with high-quality random number 
    generation, and provides tools for managing high-performance computing (HPC) array jobs
    submitted to schedulers such as SLURM. For a pedagogical introduction to the package see
    Sigal and Chalmers (2016) <doi:10.1080/10691898.2016.1246953>. For a more in-depth overview of 
    the package and its design philosophy see Chalmers and Adkins (2020) <doi:10.20982/tqmp.16.4.p248>.
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: methods, testthat, parallel, parallelly, dplyr, sessioninfo,
        beepr, pbapply (>= 1.3-0), future, future.apply, progressr,
        R.utils, codetools, clipr, e1071, qs2, stats
Suggests: knitr, ggplot2, tidyr, purrr, shiny, copula, extraDistr,
        renv, cli, job, future.batchtools, FrF2, rmarkdown,
        RPushbullet, mirai, httr
License: GPL (>= 2)
ByteCompile: yes
LazyData: true
URL: http://philchalmers.github.io/SimDesign/,
        https://github.com/philchalmers/SimDesign/wiki
Encoding: UTF-8
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-08-29 18:16:21 UTC; phil
Author: Phil Chalmers [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5332-2810>),
  Matthew Sigal [ctb],
  Ogreden Oguzhan [ctb],
  Mikko Rönkkö [aut],
  Moritz Ketzer [ctb]
Maintainer: Phil Chalmers <rphilip.chalmers@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-30 05:10:45 UTC
