# Generated by roxygen2: do not edit by hand

S3method("[",Design)
S3method(notify,PushbulletNotifier)
S3method(notify,TelegramNotifier)
S3method(plot,PBA)
S3method(plot,RM)
S3method(plot,SimSolve)
S3method(print,Design)
S3method(print,PBA)
S3method(print,RM)
S3method(print,SFA)
S3method(print,SimDesign)
S3method(print,bybye)
S3method(rbind,Design)
S3method(rbind,SimDesign)
S3method(select,bybye)
S3method(summary,SimDesign)
S3method(summary,SimSolve)
export(AnalyseIf)
export(Attach)
export(Bradley1978)
export(CC)
export(CheckPackages)
export(ECR)
export(EDR)
export(ERR)
export(GenerateIf)
export(IRMSE)
export(MAE)
export(MSRSE)
export(PBA)
export(RAB)
export(RD)
export(RE)
export(RMSD)
export(RMSE)
export(RSE)
export(RobbinsMonro)
export(SFA)
export(Serlin2000)
export(SimAnova)
export(SimCheck)
export(SimClean)
export(SimCollect)
export(SimErrors)
export(SimExtract)
export(SimFunctions)
export(SimRead)
export(SimResults)
export(SimShiny)
export(SimSolve)
export(SimWarnings)
export(addMissing)
export(add_missing)
export(aggregate_simulations)
export(arrange)
export(bias)
export(bootPredict)
export(boot_predict)
export(clusterSetRNGSubStream)
export(colSDs)
export(colVars)
export(createDesign)
export(descript)
export(expandDesign)
export(expandReplications)
export(genSeeds)
export(getArrayID)
export(get_descriptFuns)
export(group_by)
export(listAvailableNotifiers)
export(manageMessages)
export(manageWarnings)
export(nc)
export(new_PushbulletNotifier)
export(new_TelegramNotifier)
export(quiet)
export(rHeadrick)
export(rValeMaurelli)
export(rbindDesign)
export(reSummarise)
export(rejectionSampling)
export(rint)
export(rinvWishart)
export(rmgh)
export(rmvnorm)
export(rmvt)
export(rtruncate)
export(runArraySimulation)
export(runSimulation)
export(select)
export(timeFormater)
import(methods)
import(parallel)
import(stats)
importFrom(R.utils,withTimeout)
importFrom(beepr,beep)
importFrom(clipr,write_clip)
importFrom(codetools,findGlobals)
importFrom(dplyr,
  arrange,
  as_tibble,
  bind_rows,
  group_by,
  mutate,
  select
)
importFrom(e1071,
  kurtosis,
  skewness
)
importFrom(future.apply,future_lapply)
importFrom(graphics,abline)
importFrom(parallelly,availableCores)
importFrom(pbapply,
  pblapply,
  pboptions
)
importFrom(progressr,progressor)
importFrom(qs2,
  qd_read,
  qd_save
)
importFrom(sessioninfo,session_info)
importFrom(testthat,
  capture_messages,
  capture_output_lines
)
importFrom(utils,
  capture.output,
  flush.console,
  head,
  methods,
  object.size,
  packageVersion,
  recover,
  sessionInfo,
  tail
)
