% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/notifications.R
\name{notify.PushbulletNotifier}
\alias{notify.PushbulletNotifier}
\title{S3 method to send notifications via Pushbullet}
\usage{
\method{notify}{PushbulletNotifier}(notifier, event, event_data)
}
\arguments{
\item{notifier}{A TelegramNotifier object created with new_TelegramNotifier()}

\item{event}{Character string indicating the notification trigger ("condition" or "complete")}

\item{event_data}{List containing context information for the notification}
}
\value{
Invisibly returns NULL
}
\description{
S3 method to send notifications via Pushbullet
}
\keyword{internal}
